<?php 

if ( file_exists(dirname( __FILE__ ) . "/settings.php") ) {
	require_once( dirname( __FILE__ ) . "/settings.php");
}		

if ( file_exists(dirname( __FILE__ ) . "/textcachelogger.php") ) {
	require_once( dirname( __FILE__ ) . "/textcachelogger.php");
}		


class TextCacheController {
	private $cachename = "";
	private $cachedir = "";
	private $logger = null;
	private $urlmd5 = "";
	private $start = null;

	public function __construct() {
		$this->start = microtime();

		$cname = "tcc";
		$isdebug = "false";

		if (file_exists(dirname( __FILE__ ) . "/settings.php")){
			$cname = TCC_DIRNAME;
			$isdebug = TCC_ISDEBUG;
		}

		$this->cachename = $cname;
		$this->cachedir =  dirname( __FILE__ ) . "/" . $cname;
		$this->logger = new TextCacheLogger($cname, $isdebug);

    }


    public function tryCache(){
    		$ret["code"] = 0;
    		$ret["html"] = "";

		$url = "@".$_SERVER["REQUEST_URI"];
		$url = str_replace("?clear=all", "", $url);
		$url = str_replace("&clear=all", "", $url);
		$url = str_replace("?clear=page", "", $url);
		$url = str_replace("&clear=page", "", $url);
		$this->urlmd5 = md5($url);

		$isCommentOrRefresh = false;
		$isClear = 0;


		if (array_key_exists("HTTP_CACHE_CONTROL", $_SERVER) == false){

		}
		else if ($_SERVER["HTTP_CACHE_CONTROL"] == "max-age=0"){
			$isCommentOrRefresh = true;
		}

		$isLogin = $this->checkLogin();



		if (!(strpos($_SERVER["REQUEST_URI"],"?clear=all") === false)){
			$isClear = 2;			
		}
		else if (!(strpos($_SERVER["REQUEST_URI"],"&clear=all") === false)){
			$isClear = 2;			
		}
		else if (!(strpos($_SERVER["REQUEST_URI"],"?clear=page") === false)){
			$isClear = 1;			
		}
		else if (!(strpos($_SERVER["REQUEST_URI"],"&clear=page") === false)){
			$isClear = 1;			
		}

		if (!$this->mkdir($this->cachedir)){
			return $ret;
		}

		if ($isClear == 0 && 
			($isCommentOrRefresh || file_exists($this->cachedir . "/" . $this->urlmd5) != true)){

			$ret["code"] = 1;
			return $ret;
		}
		else if ($isClear == 0 && $isLogin == true){
			//if you login ,not cache, Because a html has admin bar of Wordpress is cached.
			$ret["code"] = 0;
			return $ret;
		}
		else if ($isClear == 2 && $isLogin == true){
			$this->clearCache(true, "");
			return $ret;
		}
		else if ($isClear == 1){
			$this->clearCache(false, $this->urlmd5);
			return $ret;
		}
		else{
			$html = $this->readCache($this->urlmd5);

			if ($html == "EXCEPTION"){
				return $ret;
			}

			$ret["code"] = 2;
			$ret["html"] = $html;
			return $ret;
		}

    }

    private function checkLogin(){
    		//$cookie = var_export($_COOKIE, true);

		foreach ($_COOKIE as $key => $value) {
	    		if (strpos($key, "wordpress_logged_in") === 0){
	    			return true;
	    		}
	    	}
	    	return false;
    }

    public function writeCache($html){
		if (is_dir($this->cachedir) == false){
    			return ;
    		}

		$fname = $this->cachedir . "/" . $this->urlmd5;

		$this->logger->debug("Write Cache Start: File = " . $fname);
		try{
			
			$handle = $this->logger->openHandle($fname, "w");
			if ($handle == false){
				$this->logger->log("Write Cache Failure(Timeout): File = " . $fname);
				return;
			}

			$data = $html;
			fwrite($handle, $data);

			fclose($handle);

			$this->logger->debug("Write Cache Complete: File = " . $fname);

		}
		catch (Exception $ex){
			$this->logger->log("Write Cache Failure: File = " . $fname);
			$this->logger->logEx($ex);
			try{
				fclose($handle);
			}
			catch(Exception $ex2){
				//ignore
			}
		}

    }

    private function readCache($md5){
		$fname = $this->cachedir . "/" . $md5;

		$this->logger->debug("Read Cache Start: File = " . $fname);

		try{
			$handle = $this->logger->openHandle($fname, "r");
			if ($handle == false){
				$this->logger->log("Read Cache Failure(Timeout): File = " . $fname);
				return "EXCEPTION";
			}

			$data = fread($handle,filesize($fname)) ; 	

			fclose($handle);
			$this->logger->debug("Read Cache Complete: File = " . $fname);

			return $data;

		}
		catch (Exception $ex){
			$this->logger->logEx2("Read Cache Failure: File = " . $fname, $ex);
			try{
				fclose($handle);
			}
			catch(Exception $ex2){
				//ignore
			}
		}		

		return "EXCEPTION";
    }



    private function clearCache($all , $md5){
		if (is_dir($this->cachedir) == false){
			return;
		}

		if ($all == true){
			$this->logger->debug("Clear Cache: Mode = all");

			$dirhandle = opendir($this->cachedir);

			while($fname = readdir($dirhandle) ){
				$this->logger->debug("Clear Cache: File = " . $fname);

				try{
					if ($fname == "tcc.log"){
						
					}
					else if ($fname == "." || $fname == ".."){

					}
					else{
						$this->logger->debug("Clear Cache Start: File = " . $fname);
						unlink($this->cachedir . "/" . $fname);
					}
				}
				catch (Exception $ex){
					//ignore
					$this->logger->logEx2("Clear Cache Failure: File = " . $fname, $ex);
				}
			}

			closedir($dirhandle);
			return;
		}

		if ($all == false && file_exists($this->cachedir . "/" . $md5) == false){
			$this->logger->debug("Clear Cache: Mode = page");
			return;
		}
		else{
			$this->logger->debug("Clear Cache: Mode = page");

			try{
				$this->logger->debug("Clear Cache Start: File = " . $md5);
				unlink($this->cachedir . "/" . $md5);
			}
			catch (Exception $ex){
				//ignore
				$this->logger->logEx2("Clear Cache Failure: File = " . $md5, $ex);
			}
		}
    }

    private function mkdir($dirname){
    		if (is_dir($dirname)){
			$this->logger->debug("Make Directory: is_dir(" . $dirname . ")");

    			return true;
    		}

    		if (mkdir($dirname)) {
			$this->logger->debug("Make Directory Complete: mkdir(" . $dirname . ")");
			return true;
		} 
		else {
			$msg = "create directory failure ,please check permission of " . $dirname ;
			$msg = $msg . " or check error log of apache or nginx ";

			$this->logger->log($msg);
			$this->logger->debug("Make Directory Failure: mkdir(" . $dirname . ")");
			return false;
		}
    }

    public function log($line){
		$this->logger->log($line);
    }
    
    public function logDebug($line){
		$this->logger->debug($line);
    }

    public function logEx($ex){
		$this->logger->logEx($ex);
    }

    public function logEnd($line){
		$this->logger->logEnd($this->start, $line);
    }

}


?>
