<?php 
if ( file_exists(dirname( __FILE__ ) . "/settings.php") ) {
	require_once( dirname( __FILE__ ) . "/settings.php");
}	

class TextCacheLogger{

	private $cachename;
	private $isdebug;

	//Retry default 100ms 5times
	const RETRY_COUNT = 5; 
	const RETRY_INTERVAL = 100000 ;  //1000000 = 1sec

	public function __construct($cachename, $isdebug) {
		$this->cachename = $cachename;
		$this->isdebug = false;

		if ($isdebug == "true"){
			$this->isdebug = true;			
		}
    }

	public function debug($line){
		if ($this->isdebug == true){
			$this->write($line);
		}
	}

	public function logEnd($start, $line){
		$end = microtime();

		$sa = ($this->getTime($end) - $this->getTime($start));
		$sa = round($sa, 5);

		$this->write("TotalTime : " . $sa . " " . $line);

	}

	public function log($line){
		$this->write($line);
	}

	public function logEx($ex){
		$line = $ex->getMessage() . "\n";
		$line = $line . $ex->getFile() . " line : " . $ex->getLine() . "\n";
		$line = $line . $ex->getTrace() . "\n";

		$this->write($line);
	}

	public function logEx2($line, $ex){
		$this->write($line);
		$this->logEx($ex);
	}
	
	private function write($rcvdata){
		$fname = dirname( __FILE__ ) . "/" . $this->cachename . "/tcc.log";


		try{
			$handle = $this->openHandle($fname, "a");
			if ($handle == false){
				$msg = "error*** Cannot open Log File = tcc.log , ";
				$msg = $msg .  "Message = " . $rcvdata . "<br />";

				if ($this->isdebug == false){
					$msg = "<span style='display:none;'>" . $msg . "</span>";
				}

				echo $msg;

				return;
			}

			$dtstr = $this->timezoneNow();

			$data = $dtstr . "\t" . $rcvdata . "\n";
			fwrite($handle, $data);

			fclose($handle);
		}
		catch (Exception $ex){
			$msg = "error***". $ex->getMessage();
			if ($this->isdebug == false){
				$msg = "<span style='display:none;'>" . $msg . "</span>";
			}
			echo $msg;

			try{
				fclose($handle);
			}
			catch(Exception $ex2){
				//ignore
				throw $ex2;
			}
		}

	}

	private function timezoneNow(){
		$orgtz = date_default_timezone_get();
		
		date_default_timezone_set("UTC");

		$t = new DateTime();

		$t->setTimezone( new DateTimeZone(TCC_TIMEZONE) );
		$rtn = $t->format("Y-m-d H:i:s");

		date_default_timezone_set($orgtz);

		return $rtn;
	}

	public function openHandle($fname , $mode){

		for ($i = 0 ; $i < self::RETRY_COUNT ; $i++){
			$handle = fopen($fname, $mode);
			if ($handle == false){
				usleep(self::RETRY_INTERVAL);
			}
			else{
				break;
			}
		}

		return $handle;		
	}

	private function getTime($tm) {
	    list($sec1, $sec2) = explode(" ", $tm);
	    return ((float)$sec1 + (float)$sec2);
	}

}


?>
